@@ -1,10 +1,15 @@
-VERSION=1.5.2
+VERSION=1.5.5
 FEDORA=fc35
 REP=`pwd`
 kinit grosjo@FEDORAPROJECT.ORG
 rpmdev-setuptree
 cat dovecot-fts-xapian.spec.in | sed -e s/FTSVERSION/${VERSION}/g > dovecot-fts-xapian.spec
 cp dovecot-fts-xapian.spec ~/rpmbuild/SPECS/
+#git clone https://github.com/grosjo/fts-xapian.git fts-xapian-${VERSION}
+#rm -rf  fts-xapian-${VERSION}/.git*
+#rm -rf  fts-xapian-${VERSION}/PAC*
+#tar -czvf ~/rpmbuild/SOURCES/dovecot-fts-xapian-${VERSION}.tar.gz fts-xapian-${VERSION}
+#rm -rf fts-xapian-${VERSION}
 wget https://github.com/grosjo/fts-xapian/archive/refs/tags/${VERSION}.tar.gz -O ~/rpmbuild/SOURCES/dovecot-fts-xapian-${VERSION}.tar.gz
 QA_RPATHS=$(( 0x0001|0x0010 )) rpmbuild --bb --nodebuginfo ~/rpmbuild/SPECS/dovecot-fts-xapian.spec
 QA_RPATHS=$(( 0x0001|0x0010 )) rpmbuild --bs --nodebuginfo ~/rpmbuild/SPECS/dovecot-fts-xapian.spec
@@ -22,8 +27,11 @@ git add dovecot-fts-xapian.spec
 git commit -m "Version $VERSION}"
 git push
 fedpkg import ../../dovecot-fts-xapian-${VERSION}-1.${FEDORA}.src.rpm
+git add .gitignore sources
+git commit -m "Version $VERSION}"
+git push
 fedpkg build
-fedpkh update
+fedpkg update
 fedpkg switch-branch f35
 git merge rawhide
 fedpkg push
@@ -39,7 +47,7 @@ git merge rawhide
 fedpkg push
 fedpkg build
 fedpkg update
-fedpkg switch-branch el8
+fedpkg switch-branch epel8
 git merge rawhide
 fedpkg push
 fedpkg build
