@@ -34,14 +34,14 @@ class XResultSet
 class XQuerySet
 {
 	private:
-		icu::Transliterator *accentsConverter;
-	public:
-		char * header;
-		char * text;
+		const char * header;
+		icu::UnicodeString * text;
 		XQuerySet ** qs;
 		Xapian::Query::op global_op;
 		bool item_neg; // for the term
 		long qsize;
+
+	public:
 		long limit;
 
 	XQuerySet()
@@ -50,8 +50,7 @@ class XQuerySet
 		limit=1;
 		header=NULL;
 		text=NULL;
-		global_op = Xapian::Query::op::OP_OR;		
-		accentsConverter=NULL;
+		global_op = Xapian::Query::op::OP_OR;
 	}
 
 	XQuerySet(Xapian::Query::op op, long l)
@@ -62,13 +61,11 @@ class XQuerySet
 		header=NULL;
 		text=NULL;
 		global_op=op;
-		accentsConverter=NULL;
 	}
 
 	~XQuerySet()
 	{
-		if(text!=NULL) { i_free(text); text=NULL; }
-		if(header!=NULL) { i_free(header); header=NULL; }
+		if(text!=NULL) { delete(text); text=NULL; }
 
 		for(long j=0;j<qsize;j++)
 		{
@@ -76,43 +73,49 @@ class XQuerySet
 		}
 		if(qsize>0) i_free(qs);
 		qsize=0; qs=NULL;
-		if(accentsConverter != NULL) delete(accentsConverter);
 	}
 
-	void add(const char * h,const char * t)
+	void add(long uid)
 	{
-		add(h,t,false);
+		std::string s = std::to_string(uid);
+		icu::UnicodeString t(s.c_str());
+		add(hdrs_emails[0],&t,false,NULL,false);
 	}
-
-	void add(const char * h,const char * t, bool is_neg)
+		
+	void add(const char * h2, icu::UnicodeString *t, bool is_neg,icu::Transliterator * ac, bool checklength=true)
 	{
-		if(h==NULL) return;
+		if(h2==NULL) return;
 		if(t==NULL) return;
 
-		icu::UnicodeString h2 = icu::UnicodeString::fromUTF8(icu::StringPiece(h));
-		icu::UnicodeString t2 = icu::UnicodeString::fromUTF8(icu::StringPiece(t));
-
-		add(&h2,&t2,is_neg);
-	}
-
-	void add(icu::UnicodeString *h, icu::UnicodeString *t, bool is_neg)
-	{
-		h->trim();
-                h->toLower();
-                if(h->length()<1) return;
+		icu::UnicodeString h(h2);
+		h.trim();
+                if(h.length()<1) return;
 
 		long i,j,k;
 		XQuerySet * q2;
 		icu::UnicodeString *r1,*r2;
-		std::string st1,st2;
 
-		t->toLower();
-		k=CHARS_SEP;
-		while(k>0)
-                {
-                        t->findAndReplace(chars_sep[k-1],CHAR_SPACE);
-                        k--;
-                }
+		if(ac!=NULL)
+		{
+			h.toLower();
+			t->toLower();
+		
+			k=CHARS_PB;
+                	while(k>0)
+                	{
+                	        t->findAndReplace(chars_pb[k-1],CHAR_KEY);
+                	        k--;
+                	}
+
+			k=CHARS_SEP;
+			while(k>0)
+                	{
+                	        t->findAndReplace(chars_sep[k-1],CHAR_SPACE);
+                	        k--;
+                	}
+			
+			ac->transliterate(*t);
+		}
 		t->trim();
 		if(t->length()<limit) return;
 	
@@ -131,44 +134,18 @@ class XQuerySet
                         {
                                 j = t->length();
                                 r1 = new icu::UnicodeString(*t,i+1,j-i-1);
-                                q2->add(h,r1,false);
+                                q2->add(h2,r1,false,NULL,true);
                                 delete(r1);
                                 t->truncate(i);
                                 t->trim();
                                 i = t->lastIndexOf(CHAR_SPACE);
                         }
-                        q2->add(h,t,false);
+                        q2->add(h2,t,false,NULL,true);
                         if(q2->count()>0) add(q2); else delete(q2);
                         return;
                 }
 
-		if(accentsConverter == NULL)
-		{
-			UErrorCode status = U_ZERO_ERROR;
-                	accentsConverter = icu::Transliterator::createInstance("NFD; [:M:] Remove; NFC", UTRANS_FORWARD, status);
-                	if(U_FAILURE(status))
-                	{
-                	        i_error("FTS Xapian: Can not allocate ICU translator (2)");
-                	        accentsConverter = NULL;
-                	}
-		}
-		if(accentsConverter != NULL) accentsConverter->transliterate(*t);
-
-		k=CHARS_PB;
-                while(k>0)
-                {
-                        t->findAndReplace(chars_pb[k-1],CHAR_KEY);
-                        k--;
-                }
-
-		st1.clear();
-                h->toUTF8String(st1);
-                char * h2 = i_strdup(st1.c_str());
-		st2.clear();
-		t->toUTF8String(st2);
-		char * t2 = i_strdup(st2.c_str());
-
-		if(strcmp(h2,XAPIAN_WILDCARD)==0)
+		if(h.compare(XAPIAN_WILDCARD)==0)
 		{
 			if(is_neg)
 			{
@@ -180,45 +157,63 @@ class XQuerySet
 			}
 			for(i=1;i<HDRS_NB;i++)
 			{
-				if(i!=XAPIAN_EXPUNGE_HEADER) q2->add(hdrs_emails[i],t2,false);
+				if(i!=XAPIAN_EXPUNGE_HEADER) q2->add(hdrs_emails[i],t,false,NULL,true);
 			}
 			add(q2);
-			i_free(h2);
-			i_free(t2);
 			return;
 		}
                 else
 		{
                         i=0;
-                        while((i<HDRS_NB) && (strcmp(h2,hdrs_emails[i])!=0))
+                        while((i<HDRS_NB) && (h.compare(hdrs_emails[i])!=0))
                         {
                                 i++;
                         }
                         if(i>=HDRS_NB)
                         {
-                                i_free(h2);
-				i_free(t2);
                                 return;
                         }
+			h2=hdrs_emails[i];
                 }
 
+		k=t->length()-fts_xapian_settings.full;
+		if(checklength && (k>0))
+		{
+			if(is_neg)
+                        {
+                                q2 = new XQuerySet(Xapian::Query::OP_AND_NOT,limit);
+                        }
+                        else 
+                        {
+                                q2 = new XQuerySet(Xapian::Query::OP_OR,limit);
+                        }
+			q2->add(h2,t,false,NULL,false);
+	
+			icu::UnicodeString sub;
+			for(j=0;j<k;j++)
+			{
+				sub.remove();
+                                t->extract(j,j+fts_xapian_settings.full,sub);
+                                q2->add(h2,&sub,false,NULL,false);
+			}
+			add(q2);
+			return;
+		}
 		if(text==NULL)
 		{
-			text=t2;
+			text=new icu::UnicodeString(*t);
 			header=h2;
 			item_neg=is_neg;
 			return;
 		}
 
-		i_free(h2); i_free(t2);
-		q2 = new XQuerySet(Xapian::Query::OP_AND,limit);
-		q2->add(h,t,is_neg);
+                q2 = new XQuerySet(Xapian::Query::OP_AND,limit);
+		q2->add(h2,t,is_neg,NULL,false);
 		add(q2);
 	}
 
 	void add(XQuerySet *q2)
 	{
-		if(fts_xapian_settings.verbose>1) i_info("FTS Xapian: XQuerySet->addQ()");
 		if(qsize<1)
 		{
 			qs=(XQuerySet **)i_malloc(sizeof(XQuerySet*));
@@ -251,7 +246,7 @@ class XQuerySet
 			s.append(header);
 			s.append(":");
 			s.append("\"");
-			s.append(text);
+			text->toUTF8String(s);
 			s.append("\"");
 			if(item_neg) s.append(")");
 		}
@@ -293,7 +288,7 @@ class XQuerySet
 			std::string s(header);
                         s.append(":");
                         s.append("\"");
-                        s.append(text);
+                        text->toUTF8String(s);
                         s.append("\"");
 
 			Xapian::QueryParser * qp = new Xapian::QueryParser();
@@ -339,9 +334,7 @@ class XQuerySet
 class XNGram
 {
 	private:
-		long hardlimit;
 		bool onlyone;
-		icu::Transliterator *accentsConverter;
 		icu::UnicodeString * prefix;
 		icu::UnicodeString * * * storage;
 		long * size;
@@ -358,10 +351,8 @@ class XNGram
 		size = 0;
 		debug = 0;
 		maxlength = 0;
-		hardlimit = XAPIAN_TERM_SIZELIMIT - pre->length();
 		onlyone = (pre->compare("XMID")==0);
 		prefix = pre;
-		accentsConverter = NULL;
 		storage = d;
 		size = asize;
 		title=t;
@@ -369,7 +360,6 @@ class XNGram
 
 	~XNGram()
 	{
-		if(accentsConverter != NULL) delete(accentsConverter);
 	}
 
 	bool isBase64(icu::UnicodeString *d)
@@ -391,13 +381,8 @@ class XNGram
 		long i,j,k;
 		icu::UnicodeString *r1,*r2;
 
-		d->toLower();
-		k=CHARS_SEP;
-                while(k>0)
-                {
-                        d->findAndReplace(chars_sep[k-1],CHAR_SPACE);
-                        k--;
-                }
+		if((*size)>XAPIAN_MAXTERMS_PERDOC) return;
+
 		d->trim();
                 i = d->lastIndexOf(CHAR_SPACE);
                 if(i>0)
@@ -411,25 +396,6 @@ class XNGram
                         return;
                 }
 
-		k=CHARS_PB;
-                while(k>0)
-                {
-			d->findAndReplace(chars_pb[k-1],CHAR_KEY);
-			k--;
-		}
-                
-		if(accentsConverter == NULL)
-                {
-                        UErrorCode status = U_ZERO_ERROR;
-                        accentsConverter = icu::Transliterator::createInstance("NFD; [:M:] Remove; NFC", UTRANS_FORWARD, status);
-                        if(U_FAILURE(status))
-                        {
-                                i_error("FTS Xapian: Can not allocate ICU translator (1)");
-                                accentsConverter = NULL;
-                        }
-                }
-                if(accentsConverter != NULL) accentsConverter->transliterate(*d);
-      
                 k = d->length();
                 if(k<fts_xapian_settings.partial) return;
 
@@ -492,24 +458,30 @@ class XNGram
 	{
 		long l,i,p;
 
+		if((*size)>XAPIAN_MAXTERMS_PERDOC) return;
+
 		d->trim();
 		l=d->length();
 		if(l<fts_xapian_settings.partial) return;
-
-		d->insert(0,*prefix);
 		
-		l = d->length();
-		if(l<=hardlimit)
+		icu::UnicodeString * st = new icu::UnicodeString(*d);
+		st->insert(0,*prefix);
+	
+		std::string s;
+		st->toUTF8String(s);
+	
+		l = st->length();
+		if(strlen(s.c_str())<XAPIAN_TERM_SIZELIMIT)
 		{
 			if((*size)<1)
 			{
 				*storage=(icu::UnicodeString **)malloc(sizeof(icu::UnicodeString *));
 				(*size)=1;
-				(*storage)[0]=new icu::UnicodeString(*d);
+				(*storage)[0]=st;
 			}
 			else
 			{
-				p=search(d,0,*size);
+				p=search(st,0,*size);
 				if(p>=0)
 				{
 					(*storage)=(icu::UnicodeString **)realloc((*storage),((*size) + 1)*sizeof(icu::UnicodeString *));
@@ -519,9 +491,10 @@ class XNGram
 						(*storage)[i]=(*storage)[i-1];
 						i--;
 					}
-					(*storage)[p]=new icu::UnicodeString(*d);
+					(*storage)[p]=st;
 					(*size)++;
 				}
+				else delete(st);
 			}
 			if(l>maxlength) { maxlength=l; }
 		}
@@ -536,7 +509,7 @@ class XDoc
 		std::vector<icu::UnicodeString *> * strings;
 		std::vector<icu::UnicodeString *> * headers;
 	public:
-		long uid,size,stems;
+		long uid,size,stems,tsize;
 		char * uterm;
 		Xapian::Document * xdoc;
  
@@ -550,6 +523,7 @@ class XDoc
 		headers->clear();
 		size=0;
 		stems=0;
+		tsize=0;
 		std::string s;
                 s.append("Q"+std::to_string(uid));
 		uterm = (char*)malloc((s.length()+1)*sizeof(char));
@@ -587,20 +561,48 @@ class XDoc
 	{
 		std::string s("Doc ");
 		s.append(std::to_string(uid));
-		s.append(" Size(text)=" + std::to_string(size));
-		s.append(" Stems=" + std::to_string(stems));
+		s.append(" #add=" + std::to_string(size));
+		s.append(" #length=" + std::to_string(tsize));
+		s.append(" #stems=" + std::to_string(stems));
 		return s;
 	}
 
-	void add(const char *h, icu::UnicodeString* t)
+	void add(const char *h, icu::UnicodeString* t, icu::Transliterator * accentsConverter, long verbose, const char * title)
 	{
 		icu::UnicodeString * prefix = new icu::UnicodeString(h);
+		prefix->trim();
 		headers->push_back(prefix);
 
 		icu::UnicodeString * t2 = new icu::UnicodeString(*t);
-		strings->push_back(t2);
+		t2->toLower();
+		long k=CHARS_SEP;
+                while(k>0)
+                {
+                        t2->findAndReplace(chars_sep[k-1],CHAR_SPACE);
+                        k--;
+                }
+		t2->trim();
+
+		k=CHARS_PB;
+                while(k>0)
+                {
+                        t2->findAndReplace(chars_pb[k-1],CHAR_KEY);
+                        k--;
+                }
+
+		accentsConverter->transliterate(*t2);
 	
+		strings->push_back(t2);
+
+		if(verbose>0) 
+		{
+			std::string s;
+			t->toUTF8String(s);
+			s= s.substr(0,100);
+			syslog(LOG_INFO,"%s %s : Adding [%s] [%s]",title,getSummary().c_str(),h,s.c_str());
+		}
 		size++;
+		tsize+=t2->length();
 	}
 
 	void populate_stems(long verbose, const char * title)
@@ -619,6 +621,7 @@ class XDoc
 				std::string s; headers->at(j-1)->toUTF8String(s);
 				syslog(LOG_INFO,"%s %s : Populate %ld / %ld Header=%s TextLength=%ld",title,getSummary().c_str(),j,k,s.c_str(),(long)strings->at(j-1)->length());
 			}
+			
 			ngram = new XNGram(headers->at(j-1),&data,&stems,title,verbose);
 			ngram->add(strings->at(j-1));
 			delete(ngram);
@@ -626,7 +629,26 @@ class XDoc
 			delete(strings->at(j-1)); strings->at(j-1)=NULL; strings->pop_back();
                 }
 		t = fts_backend_xapian_current_time() -t;
-		if(verbose>0) syslog(LOG_INFO,"%s %s : Done populating in %ld ms (%ld stems/sec)",title,getSummary().c_str(), t, (long)(stems*1000.0/t));
+		if(verbose>0) 
+		{
+			syslog(LOG_INFO,"%s %s : Done populating in %ld ms (%ld stems/sec)",title,getSummary().c_str(), t, (long)(stems*1000.0/t));
+			std::string s,s2;
+			for(long i=0; i<10 && i<stems;i++)
+			{
+				s2.clear();
+				data[i]->toUTF8String(s2);
+				s+=" ";
+				s+=s2;
+			}
+			for(long i=stems-1; i>=0 && i>stems-10; i--)
+			{
+                                s2.clear();
+                                data[i]->toUTF8String(s2);
+                                s+=" ";
+                                s+=s2;
+                        }
+			syslog(LOG_INFO, "STEMS populated : %ld [%s]",stems,s.c_str());
+		}
 	}
 
 	void create_document(long verbose, const char * title)
@@ -1054,7 +1076,7 @@ static bool fts_backend_xapian_push(struct xapian_fts_backend *backend, const ch
 	return false;
 }
 
-static void fts_backend_xapian_close_db(Xapian::WritableDatabase * dbw,char * dbpath,char * boxname, uintmax_t uid, uintmax_t gid, long verbose, bool sysl)
+static void fts_backend_xapian_close_db(Xapian::WritableDatabase * dbw,char * dbpath,char * boxname, /* uintmax_t uid, uintmax_t gid, */ long verbose, bool sysl)
 {
 	long t = fts_backend_xapian_current_time();
 
@@ -1085,7 +1107,7 @@ static void fts_backend_xapian_close_db(Xapian::WritableDatabase * dbw,char * db
 		if(sysl) syslog(LOG_INFO,"FTS Xapian : DB (%s,%s) closed in %ld ms",boxname,dbpath,t);
 		else i_info("FTS Xapian : DB (%s,%s) closed in %ld ms",boxname,dbpath,t);
 	}
-
+/*
 	{
 		std::string iamglass(dbpath);
 		iamglass.append("/iamglass");
@@ -1100,6 +1122,7 @@ static void fts_backend_xapian_close_db(Xapian::WritableDatabase * dbw,char * db
 			else i_error("FTS Xapian : Can not chown %s",iamglass.c_str());	
 		}
 	}
+*/
 	free(dbpath);
 	free(boxname);
 }
@@ -1134,7 +1157,7 @@ static void fts_backend_xapian_close(struct xapian_fts_backend *backend, const c
 		}
 		else
 		{
-			if(fts_xapian_settings.verbose>0) i_info("FTS Xapian : Waiting for #%ld (Sleep4) : %s",i,(backend->threads)[i]->getSummary().c_str());
+			if(fts_xapian_settings.verbose>1) i_info("FTS Xapian : Waiting for #%ld (Sleep4) : %s",i,(backend->threads)[i]->getSummary().c_str());
 			std::this_thread::sleep_for(XSLEEP);
 			i++;
 		}
@@ -1147,19 +1170,19 @@ static void fts_backend_xapian_close(struct xapian_fts_backend *backend, const c
 	{
 		char * dbpath = (char*) malloc(sizeof(char)*(strlen(backend->db)+1));	
 		strcpy(dbpath,backend->db);
-		struct stat fileinfo;
-		stat(dbpath,&fileinfo);
+		/* struct stat fileinfo;
+		stat(dbpath,&fileinfo); */
 		char * boxname = (char*) malloc(sizeof(char)*(strlen(backend->boxname)+1));
 		strcpy(boxname,backend->boxname);
 		try
         	{
 			if(fts_xapian_settings.detach)
 			{
-				(new std::thread(fts_backend_xapian_close_db,backend->dbw,dbpath,boxname,fileinfo.st_uid,fileinfo.st_gid,fts_xapian_settings.verbose,true))->detach();
+				(new std::thread(fts_backend_xapian_close_db,backend->dbw,dbpath,boxname,/* fileinfo.st_uid,fileinfo.st_gid,*/ fts_xapian_settings.verbose,true))->detach();
 			}
 			else
 			{
-				fts_backend_xapian_close_db(backend->dbw,dbpath,boxname,fileinfo.st_uid,fileinfo.st_gid,fts_xapian_settings.verbose,false);
+				fts_backend_xapian_close_db(backend->dbw,dbpath,boxname,/* fileinfo.st_uid,fileinfo.st_gid,*/ fts_xapian_settings.verbose,false);
 			}
         	}
         	catch(std::exception e)
@@ -1335,7 +1358,7 @@ static int fts_backend_xapian_set_box(struct xapian_fts_backend *backend, struct
 	return 0;
 }
 
-static void fts_backend_xapian_build_qs(XQuerySet * qs, struct mail_search_arg *a)
+static void fts_backend_xapian_build_qs(XQuerySet * qs, struct mail_search_arg *a,icu::Transliterator * ac)
 {
 	const char * hdr;
 
@@ -1379,7 +1402,7 @@ static void fts_backend_xapian_build_qs(XQuerySet * qs, struct mail_search_arg *
 			{
 				q2 = new XQuerySet(Xapian::Query::OP_OR,qs->limit);
 			}
-			fts_backend_xapian_build_qs(q2,a->value.subargs);
+			fts_backend_xapian_build_qs(q2,a->value.subargs,ac);
 			if(q2->count()>0)
 			{
 				qs->add(q2);
@@ -1401,8 +1424,9 @@ static void fts_backend_xapian_build_qs(XQuerySet * qs, struct mail_search_arg *
 				}
 				i++;
 			}
+			icu::UnicodeString t(a->value.str);
 			char * h = i_strdup(f2.c_str());
-			qs->add(h,a->value.str,a->match_not);
+			qs->add(h,&t,a->match_not,ac,true);
 			i_free(h);
 		}
 		a->match_always=true;
@@ -1426,7 +1450,7 @@ bool fts_backend_xapian_index(struct xapian_fts_backend *backend, const char* fi
 	if(i>=HDRS_NB) i=HDRS_NB-1;
 	const char * h = hdrs_xapian[i];
 
-	backend->docs->back()->add(h,data);
+	backend->docs->back()->add(h,data,backend->accentsConverter,fts_xapian_settings.verbose,"FTS Xapian: fts_backend_xapian_index");
 
 	if(fts_xapian_settings.verbose>1) i_info("FTS Xapian: fts_backend_xapian_index %s done",field);
 
