@@ -54,6 +54,8 @@ struct xapian_fts_backend
 	long total_added_docs;
 	long batch;
 	long start_time;
+
+	icu::Transliterator * accentsConverter;
 };
 
 struct xapian_fts_backend_update_context
@@ -107,6 +109,15 @@ static int fts_backend_xapian_init(struct fts_backend *_backend, const char **er
 
 	if(fts_backend_xapian_set_path(backend)<0) return -1;
 
+	UErrorCode status = U_ZERO_ERROR;
+	backend->accentsConverter = icu::Transliterator::createInstance("NFD; [:M:] Remove; NFC", UTRANS_FORWARD, status);
+	if(U_FAILURE(status))
+        { 
+		i_error("FTS Xapian: Can not allocate ICU translator (2)");
+                backend->accentsConverter = NULL;
+		return -1;
+	}
+
 	openlog("xapian-docswriter",0,LOG_MAIL);
 
         if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: Starting with partial=%ld full=%ld verbose=%ld lowmemory=%ld MB vs freemem=%ld MB", fts_xapian_settings.partial,fts_xapian_settings.full,fts_xapian_settings.verbose,fts_xapian_settings.lowmemory, long(fts_backend_xapian_get_free_memory()/1024.0));
@@ -131,6 +142,9 @@ static void fts_backend_xapian_deinit(struct fts_backend *_backend)
 	if(backend->path != NULL) i_free(backend->path);
 	backend->path = NULL;
 
+	if(backend->accentsConverter != NULL) i_free(backend->accentsConverter);
+	backend->accentsConverter = NULL;
+
 	i_free(backend);
 
 	closelog();
@@ -505,15 +519,15 @@ static int fts_backend_xapian_optimize(struct fts_backend *_backend)
 	char *zErrMsg = 0;
 	XResultSet * result = NULL;
 	Xapian::docid docid =0;
-	struct stat fileinfo;
+//	struct stat fileinfo;
 	while ((dp = readdir(dirp)) != NULL)
 	{
 		if((dp->d_type == DT_DIR) && (strncmp(dp->d_name,"db_",3)==0))
 		{
 			uids.clear();
-			s = i_strdup_printf("%s/%s",backend->path,dp->d_name);
+			/* s = i_strdup_printf("%s/%s",backend->path,dp->d_name);
 			stat(s, &fileinfo);
-			i_free(s);
+			i_free(s);*/
 			s = i_strdup_printf("%s/%s_exp.db",backend->path,dp->d_name);
 			i_info("FTS Xapian: Optimize (1) %s : Checking expunges",s);
 			if(sqlite3_open(s,&expdb) == SQLITE_OK)
@@ -560,9 +574,7 @@ static int fts_backend_xapian_optimize(struct fts_backend *_backend)
 						uid=uids[n];
 						if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: Optimize (5) Removing DOC UID=%d",uid);
 						XQuerySet * xq = new XQuerySet();
-						char *u = i_strdup_printf("%d",uid);
-						xq->add("uid",u);
-						i_free(u);
+						xq->add(uid);
 						result=fts_backend_xapian_query(db,xq,1);
 						docid=0; 
 						if((result!=NULL) && (result->size>0))
@@ -588,7 +600,7 @@ static int fts_backend_xapian_optimize(struct fts_backend *_backend)
 						else if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: Optimize UID=%d (DOCID=%d) inexistent",uid,docid);
 						if(result!=NULL) { delete(result); result=NULL; }
 						delete(xq);
-						u = i_strdup_printf("delete from docs where ID=%d",uid);
+						char * u = i_strdup_printf("delete from docs where ID=%d",uid);
 						if (sqlite3_exec(expdb,u,NULL,0,&zErrMsg) != SQLITE_OK )
 						{
 							i_error("FTS Xapian : Optimize Sqlite error %s",zErrMsg);
@@ -603,11 +615,11 @@ static int fts_backend_xapian_optimize(struct fts_backend *_backend)
 					strcpy(s2,"fts_optimize");
 					if(fts_xapian_settings.detach)
 					{
-						(new std::thread(fts_backend_xapian_close_db,db,s1,s2,fileinfo.st_uid,fileinfo.st_gid,fts_xapian_settings.verbose,true))->detach();
+						(new std::thread(fts_backend_xapian_close_db,db,s1,s2,/*fileinfo.st_uid,fileinfo.st_gid,*/fts_xapian_settings.verbose,true))->detach();
 					}
 					else
 					{
-						fts_backend_xapian_close_db(db,s1,s2,fileinfo.st_uid,fileinfo.st_gid,fts_xapian_settings.verbose,false);
+						fts_backend_xapian_close_db(db,s1,s2,/*fileinfo.st_uid,fileinfo.st_gid,*/fts_xapian_settings.verbose,false);
 					}
 				}
 				catch(Xapian::Error e)
@@ -707,7 +719,7 @@ static int fts_backend_xapian_lookup(struct fts_backend *_backend, struct mailbo
 		qs = new XQuerySet(Xapian::Query::OP_OR,fts_xapian_settings.partial);
 	}
 
-	fts_backend_xapian_build_qs(qs,args);
+	fts_backend_xapian_build_qs(qs,args,backend->accentsConverter);
 
 	XResultSet * r=fts_backend_xapian_query(dbr,qs);
 
