@@ -29,6 +29,7 @@
 
 #define XAPIAN_FILE_PREFIX "xapian-indexes"
 #define XAPIAN_TERM_SIZELIMIT 245L
+#define XAPIAN_MAXTERMS_PERDOC 50000L
 #define XAPIAN_THREAD_SIZE 3L
 #define XAPIAN_WRITING_CACHE 2000L
 
@@ -37,9 +38,9 @@
 
 #define XAPIAN_MAX_SEC_WAIT 15L
 
-#define HDRS_NB 10
-static const char * hdrs_emails[HDRS_NB] = { "uid", "subject", "from", "to",  "cc",  "bcc",  "messageid", "listid", "body", ""  };
-static const char * hdrs_xapian[HDRS_NB] = { "Q",   "S",       "A",    "XTO", "XCC", "XBCC", "XMID",      "XLIST",  "XBDY", "XBDY" };
+#define HDRS_NB 11
+static const char * hdrs_emails[HDRS_NB] = { "uid", "subject", "from", "to",  "cc",  "bcc",  "messageid", "listid", "body", "contenttype", ""  };
+static const char * hdrs_xapian[HDRS_NB] = { "Q",   "S",       "A",    "XTO", "XCC", "XBCC", "XMID",      "XLIST",  "XBDY", "XCT", "XBDY" };
 static const char * createTable = "CREATE TABLE IF NOT EXISTS docs(ID INT PRIMARY KEY NOT NULL);";
 static const char * selectUIDs = "select ID from docs;";
 #define CHAR_KEY "_"
